from osbot_utils.type_safe.Type_Safe import Type_Safe


class Schema__Status__Upload_Dependency__To_S3(Type_Safe):
    duration__install_locally : float
    duration__upload_to_s3    : float
    result__already_existed   : bool
    result__install_locally   : bool
    result__upload_to_s3      : bool



from osbot_aws.aws.bedrock.models.titan.base_classes.Amazon_Titan__Text import Amazon_Titan__Text

# for details on this model see https://docs.aws.amazon.com/bedrock/latest/userguide/model-parameters-titan-text.html

class Amazon_Titan_Tg1_Large(Amazon_Titan__Text):
    model_id     : str = 'amazon.titan-tg1-large'



from typing                                                                         import List
from osbot_utils.type_safe.Type_Safe                                                import Type_Safe
from osbot_aws.aws.lambda_.schemas.Safe_Str__File__Path__Python_Package             import Safe_Str__File__Path__Python_Package
from osbot_aws.aws.lambda_.schemas.Safe_Str__File__Name__Python_Package             import Safe_Str__File__Name__Python_Package
from osbot_utils.type_safe.primitives.domains.identifiers.safe_int.Timestamp_Now             import Timestamp_Now
from osbot_utils.type_safe.primitives.domains.files.safe_str.Safe_Str__File__Path   import Safe_Str__File__Path

class Schema__Lambda__Dependency__Local_Install__Data(Type_Safe):
    package_name        : Safe_Str__File__Name__Python_Package  = None
    target_path         : Safe_Str__File__Path__Python_Package  = None
    install_data        : dict                                  = None
    installed_files     : List[Safe_Str__File__Path]
    time_stamp          : Timestamp_Now
    duration            : float                                 = None



from osbot_aws.aws.lambda_.dependencies.Lambda__Dependency__Base                    import Lambda__Dependency__Base
from osbot_utils.type_safe.primitives.domains.identifiers.Safe_Id                  import Safe_Id
from osbot_utils.utils.Zip                                                          import zip_folder_to_bytes
from osbot_aws.aws.lambda_.schemas.Safe_Str__File__Path__Python_Package             import Safe_Str__File__Path__Python_Package
from osbot_aws.aws.lambda_.schemas.Schema__Lambda__Dependency__Local_Install__Data  import Schema__Lambda__Dependency__Local_Install__Data
from osbot_utils.utils.Json                                                         import json_file_load, json_file_save
from osbot_utils.type_safe.primitives.domains.files.safe_str.Safe_Str__File__Path      import Safe_Str__File__Path
from osbot_utils.helpers.duration.decorators.capture_duration                       import capture_duration
from osbot_utils.utils.Files                                                        import path_combine, files_list__virtual_paths, file_exists, file_delete, folder_delete_recursively, current_temp_folder, create_folder
from osbot_utils.utils.Process                                                      import Process

FOLDER_NAME__LAMBDA_DEPENDENCIES_STORAGE  = Safe_Id('osbot-aws__lambda-dependencies-storage')

class Lambda__Dependency__Local(Lambda__Dependency__Base):
    target_aws_lambda: bool = True                                      # todo: rename this var to a better name

    def base_folder(self):
        return self.temp_folder

    def folder__bucket(self):
        return path_combine(self.folder__lambda_dependencies_storage(), self.s3__bucket__name())

    def folder__lambda_dependencies_storage(self):
        return path_combine(self.base_folder(), FOLDER_NAME__LAMBDA_DEPENDENCIES_STORAGE)

    def s3__bucket__name(self):
        return self.aws_config.lambda_s3_bucket()

    def delete(self):
        dependency_folder__delete = folder_delete_recursively(self.path())
        install_data_file__delete = self.install__data__delete()
        return dependency_folder__delete and install_data_file__delete

    def exists(self):
        return self.install__data__exists()

    def files(self):
        return files_list__virtual_paths(self.path())

    def files__zipped(self):
        root_dir        = self.path()
        files_to_ignore = '.DS_Store'
        return zip_folder_to_bytes(root_dir=root_dir, files_to_ignore=files_to_ignore)


    def install(self):
        local_install_data = self.install__data()
        if local_install_data is None:
            with capture_duration() as duration__install:
                target_path = self.path()
                args = ['install']
                if self.target_aws_lambda:
                    args.extend(['--platform','manylinux1_x86_64', '--only-binary=:all:'])
                args.extend(['-t', target_path, self.package_name])
                install_data  = Process.run('pip3', args)
                installed_files = self.files()
            kwargs = dict(package_name       = self.package_name        ,
                          target_path        = target_path              ,
                          install_data       = install_data             ,
                          installed_files    = installed_files          ,
                          duration           = duration__install.seconds)

            local_install_data = Schema__Lambda__Dependency__Local_Install__Data (**kwargs)
            self.install__data__save(local_install_data)
        return local_install_data

    def install__data(self) -> Schema__Lambda__Dependency__Local_Install__Data:
        if self.install__data__exists():
            json_data   = json_file_load(self.install__data__path())
            local_install_data = Schema__Lambda__Dependency__Local_Install__Data.from_json(json_data)
            return local_install_data

    def install__data__delete(self) -> bool:
        return file_delete(self.install__data__path())

    def install__data__exists(self) -> bool:
        return file_exists(self.install__data__path())

    def install__data__path(self) -> Safe_Str__File__Path:
        return self.path() + ".json"

    def install__data__save(self, local_install_data: Schema__Lambda__Dependency__Local_Install__Data) -> Safe_Str__File__Path:
        json_data = local_install_data.json()
        file_path = self.install__data__path()
        return json_file_save(json_data, path=file_path)

    def path(self) -> Safe_Str__File__Path__Python_Package:
        local__dependency__path = path_combine(self.folder__bucket(), self.package_name)
        return Safe_Str__File__Path__Python_Package(local__dependency__path)



    def setup(self):
        create_folder(self.folder__bucket())
        return self


from osbot_utils.utils.Zip                                              import zip_bytes__files, zip_bytes__files_paths
from osbot_aws.aws.lambda_.dependencies.Lambda__Dependency__Base        import Lambda__Dependency__Base
from osbot_aws.aws.lambda_.schemas.Safe_Str__File__Path__Python_Package import Safe_Str__File__Path__Python_Package
from osbot_aws.aws.s3.S3                                                import S3, S3_FILES_CONTENT_TYPES

FOLDER_NAME__LAMBDA_DEPENDENCIES = Safe_Str__File__Path__Python_Package('lambdas-dependencies')

class Lambda__Dependency__S3(Lambda__Dependency__Base):
    s3           : S3

    def bucket__name(self):
        return self.aws_config.lambda_s3_bucket()

    def bucket__exists(self):
        return self.s3.bucket_exists(self.bucket__name())

    def bytes(self):
        return self.s3.file_bytes(self.bucket__name(), self.path())

    def delete(self):
        return self.s3.file_delete(self.bucket__name(), self.path())

    def exists(self):
        return self.s3.file_exists(self.bucket__name(), self.path())

    def files(self):
        return zip_bytes__files(self.bytes())

    def files__paths(self):
        return zip_bytes__files_paths(self.bytes())

    def folder_name__lambda_dependencies(self):
        return FOLDER_NAME__LAMBDA_DEPENDENCIES

    def metadata(self):
        return self.s3.file_metadata(self.bucket__name(), self.path())

    def path(self) -> Safe_Str__File__Path__Python_Package:
        return self.folder_name__lambda_dependencies() + f"/{self.package_name}.zip"

    def setup(self):
        bucket_name = self.bucket__name()
        if self.bucket__exists() is False:
            region_name = self.aws_config.region_name()
            self.s3.bucket_create(bucket= bucket_name, region=region_name)
        return self

    def upload(self, file_bytes):
        kwargs       = dict(file_bytes   = file_bytes,
                            bucket       = self.bucket__name(),
                            key          = self.path(),
                            metadata     = {'data_type': 'zip-folder-for-lambda-function'},
                            content_type = S3_FILES_CONTENT_TYPES.get('.zip'))
        self.s3.file_create_from_bytes(**kwargs)
        return kwargs




import sys
from osbot_utils.utils.Zip                                       import zip_bytes__extract_to_folder
from osbot_utils.type_safe.primitives.domains.identifiers.Safe_Id                                 import Safe_Id
from osbot_utils.utils.Files                                     import path_combine, folder_exists, files_list__virtual_paths
from osbot_aws.aws.lambda_.dependencies.Lambda__Dependency__Base import Lambda__Dependency__Base
from osbot_aws.aws.lambda_.dependencies.Lambda__Dependency__S3   import Lambda__Dependency__S3

FOLDER_NAME__LAMBDA_DEPENDENCIES__INSIDE_LAMBDA  = Safe_Id('osbot-aws__lambda-dependencies__inside-lambda')

class Lambda__Dependency__Inside_Lambda(Lambda__Dependency__Base):
    skip__when_not_inside_lambda: bool                   = True
    lambda_dependency_s3        : Lambda__Dependency__S3 = None

    def __init__(self, **kwargs):
        super().__init__(**kwargs)
        self.lambda_dependency_s3 = Lambda__Dependency__S3(package_name=self.package_name)

    def dependencies_folder(self):
        return path_combine(self.temp_folder, FOLDER_NAME__LAMBDA_DEPENDENCIES__INSIDE_LAMBDA)

    def exists(self):
        return folder_exists(self.folder())

    def enabled(self):
        return self.exists() and self.system_path__contains_folder()

    def files(self):
        return files_list__virtual_paths(self.folder())

    def folder(self):
        return path_combine(self.dependencies_folder(), self.package_name)

        #s3_key     = 'lambdas-dependencies/{0}.zip'.format(target)
        #tmp_dir    = Files.path_combine('/tmp/lambdas-dependencies', target)

    def load(self):
        target_folder = self.folder()
        if self.exists() is False:
            with self.lambda_dependency_s3 as _:
                 if _.exists() is False:
                    raise RuntimeError(f"in Lambda__Dependency__Load the requested package {self.package_name} "
                                       f"was not found in: {_.bucket__name()} : {_.path()}")
                 zip_bytes__extract_to_folder(zip_bytes=_.bytes(), target_folder=target_folder)
        self.system_path__add_folder()
        return self.enabled()

    def system_path(self):
        return sys.path

    def system_path__add_folder(self):
        if self.system_path__contains_folder() is False:
            sys.path.append(self.folder())
            return True
        return False

    def system_path__contains_folder(self):
        return self.folder() in self.system_path()



from datetime                                                       import datetime
from osbot_utils.type_safe.type_safe_core.decorators.type_safe      import type_safe
from osbot_utils.type_safe.Type_Safe                                import Type_Safe
from osbot_utils.type_safe.primitives.domains.identifiers.Safe_Id  import Safe_Id
from osbot_utils.utils.Misc                                         import utc_now, date_today

S3_PATH__WHEN_BLOCK_SIZE   = 5

class S3__Key_Generator(Type_Safe):
    root_folder        : str  = None
    server_name        : str  = None
    use_when           : bool = True
    use_date           : bool = True
    use_hours          : bool = True
    use_minutes        : bool = True
    save_as_gz         : bool = False
    split_when         : bool = False
    s3_path_block_size : int = S3_PATH__WHEN_BLOCK_SIZE

    def calculate_minute_block(self, minute):
        block_size   = self.s3_path_block_size                                  # get the block size in minutes (configurable)
        minute_block = f"{(int(minute) // block_size) * block_size:02d}"        # Calculate the block using the configurable block size
        return minute_block

    @type_safe
    def create__for_area_and_file_id(self, area: Safe_Id, file_id: Safe_Id):
        path_elements = self.create_path_elements__from_when(area=area)
        s3_key = self.create_s3_key(path_elements=path_elements, file_id=file_id)
        return s3_key

    def create_path_elements__for_server(self):
        path_elements = []
        if self.root_folder: path_elements.append(self.root_folder)
        if self.server_name: path_elements.append(self.server_name)
        return path_elements

    def create_path_elements__from_when(self, when=None, area: Safe_Id = None):
        path_elements = self.create_path_elements__for_server()
        if area:
            path_elements.append(area)
        if self.use_when:
            if not when:
                when = self.path__for_date_time__now_utc()
            if when:                                            # for the cases when path__for_date_time__now_utc returns and empty value
                path_elements.append(when)



        return path_elements

    def create_s3_key(self, path_elements, file_id):
        path_elements.append(file_id)
        s3_key = '/'.join(path_elements) + '.json'
        if self.save_as_gz:
            s3_key += ".gz"
        return s3_key

    def create_s3_folder(self, path_elements):
        s3_folder = '/'.join(path_elements)
        return s3_folder

    def path__for_date_time__now_utc(self):
        return self.path__for_date_time(utc_now())

    def path__for_date_time(self, date_time: datetime):
        minute       = date_time.minute
        date_path    = date_time.strftime('%Y-%m-%d')                          # Format the date as YYYY-MM-DD
        hour_path    = date_time.strftime('%H')                                # Format the hour
        minute_block = self.calculate_minute_block(minute)
        path_components = []
        if self.use_date:
            if self.split_when:
                path_components.extend(date_path.split('-'))
            else:
                path_components.append(date_path   )
        if self.use_hours:
            path_components.append(hour_path   )
        if self.use_minutes:
            path_components.append(minute_block)
        s3_path = '/'.join(path_components)
        return s3_path

    def s3_key(self, area: Safe_Id, file_id: Safe_Id):
        s3_key = self.create__for_area_and_file_id(area=area,file_id=file_id)
        return s3_key

    def s3_folder__for_day(self, day=None):
        path_elements = self.create_path_elements__for_server()
        if day is None:
            day = date_today()
        path_elements.append(day)
        return self.create_s3_folder(path_elements)

    @type_safe
    def s3_folder__for_area(self, area: Safe_Id):
        path_elements = self.create_path_elements__for_server()
        path_elements.append(area)
        return self.create_s3_folder(path_elements)

    @type_safe
    def s3_folder__for_date_time(self, date_time: datetime):
        return self.path__for_date_time(date_time)



import re
from osbot_utils.type_safe.primitives.domains.files.safe_str.Safe_Str__File__Name import Safe_Str__File__Name

TYPE_SAFE_STR__FILE__PATH__PYTHON_PACKAGE__REGEX      = re.compile(r'[^a-zA-Z0-9_\-. =]')  # Allow alphanumerics, underscores, hyphens, dots, slashes, spaces and equal

class Safe_Str__File__Name__Python_Package(Safe_Str__File__Name):
    regex = TYPE_SAFE_STR__FILE__PATH__PYTHON_PACKAGE__REGEX



from osbot_aws.aws.session.Session__Kwargs__S3      import Session__Kwargs__S3
from osbot_utils.type_safe.Type_Safe                import Type_Safe
from osbot_utils.decorators.methods.cache_on_self   import cache_on_self

ENV_NAME__USE_MINIO_AS_S3            = 'USE_MINIO_AS_S3'
S3_DB_BASE__BUCKET_NAME__SUFFIX      = "osbot-aws"
S3_DB_BASE__BUCKET_NAME__PREFIX: str = 'unknown-service'
S3_DB_BASE__SERVER_NAME              = 'unknown-server'
S3_FOLDER__TEMP_FILE_UPLOADS         = 'temp_file_uploads'
DEFAULT__LOCAL_STACK__TARGET_SERVER = 'http://localhost:4566'

class S3__DB_Base(Type_Safe):
    use_minio                      : bool = False
    bucket_name__suffix            : str  = S3_DB_BASE__BUCKET_NAME__SUFFIX     # todo: change type from str to Safe_Str_*
    bucket_name__prefix            : str  = S3_DB_BASE__BUCKET_NAME__PREFIX     # todo: change type from str to Safe_Str_*
    bucket_name__insert_account_id : bool = True
    bucket_versioning              : bool = True
    save_as_gz                     : bool = False
    server_name                    : str  = S3_DB_BASE__SERVER_NAME             # todo: change type from str to Safe_Id (or equivalent)
    session_kwargs__s3             : Session__Kwargs__S3

    def bucket_name(self):
        return self.s3_bucket()

    @cache_on_self
    def s3(self):
        from osbot_aws.aws.s3.S3   import S3
        from osbot_utils.utils.Env import get_env

        if self.use_minio or get_env(ENV_NAME__USE_MINIO_AS_S3) == 'True':
            from osbot_aws.aws.s3.S3__Minio import S3__Minio

            self.use_minio = True                                               # todo: remove this hardcoded minio dependency, since in most cases we are using LocalStack
            s3 = S3__Minio().s3()
        else:
            s3 = S3(session_kwargs__s3=self.session_kwargs__s3)
        return s3

    @cache_on_self
    def s3_bucket(self):
        from osbot_aws.AWS_Config             import aws_config
        from osbot_aws.utils.AWS_Sanitization import str_to_valid_s3_bucket_name

        separator  = '-'
        bucket_name = ''
        if self.bucket_name__prefix:
            bucket_name += f'{self.bucket_name__prefix}{separator}'                             # todo: see if creating a Safe_Str__Bucket_Name isn't going to be better here
        if self.bucket_name__insert_account_id:
            account_id  = aws_config.account_id()
            bucket_name += f'{account_id}{separator}'
        bucket_name += self.bucket_name__suffix
        return str_to_valid_s3_bucket_name(bucket_name)                                           # make sure it is a valid s3 bucket name

    def s3_bucket__temp_data(self):                                                               # override to control the bucket used for temp data (note that it is the caller responsibility to make sure the bucket exists)
        return self.s3_bucket()

    def s3_file_bytes(self, s3_key, version_id=None):
        return self.s3().file_bytes(self.s3_bucket(), s3_key, version_id)

    def s3_file_contents(self, s3_key, version_id=None):
        return self.s3().file_contents(self.s3_bucket(), s3_key, version_id=version_id)

    def s3_file_contents_json(self, s3_key, version_id=None):
        from osbot_utils.utils.Json import json_parse

        return json_parse(self.s3_file_contents(s3_key, version_id=version_id))

    def s3_file_contents_obj(self, s3_key, version_id=None):                                                     # Convert S3 file contents to a Python object using str_to_obj
        from osbot_utils.utils.Objects import str_to_obj
        return str_to_obj(self.s3_file_contents(s3_key, version_id=version_id))

    def s3_file_data(self, s3_key, version_id=None):
        from osbot_utils.utils.Files import file_extension
        from osbot_utils.utils.Json  import gz_to_json

        if self.s3_file_exists(s3_key):
            if self.save_as_gz and file_extension(s3_key) == '.gz':
                data_gz = self.s3_file_bytes(s3_key,version_id=version_id)
                data    = gz_to_json(data_gz)
            else:
                data = self.s3_file_contents_json(s3_key,version_id=version_id)
            return data

    def s3_file_delete(self, s3_key):
        kwargs = dict(bucket = self.s3_bucket(),
                      key    = s3_key          )
        return self.s3().file_delete(**kwargs)

    def s3_file_exists(self, s3_key):
        bucket = self.s3_bucket()
        return self.s3().file_exists(bucket, s3_key)

    def s3_file_info(self, s3_key):
        return self.s3().file_details(self.s3_bucket(), s3_key)

    def s3_file_metadata(self, s3_key):
        return self.s3().file_metadata(self.s3_bucket(), s3_key)

    def s3_file_set_metadata(self, s3_key, metadata):
        return self.s3().file_metadata_update(self.s3_bucket(), s3_key, metadata)

    def s3_files_delete(self, s3_keys):                             # Delete multiple S3 files, returning True only if all deletions succeed
        result = True
        for s3_key in s3_keys:
            if not self.s3_file_delete(s3_key):
                result = False
        return result

    def s3_folder_contents(self, folder, return_full_path=False):
        return self.s3().folder_contents(s3_bucket=self.s3_bucket(), parent_folder=folder, return_full_path=return_full_path)

    def s3_folder_files(self, folder='', return_full_path=False, include_sub_folders=False):
        if include_sub_folders:
            return self.s3().find_files(bucket=self.s3_bucket(), prefix=folder)
        else:
            return self.s3().folder_files(s3_bucket=self.s3_bucket(), parent_folder=folder, return_full_path=return_full_path)

    def s3_folder_files__all(self, folder='', full_path=True):                                      # Get all files in a folder, with option to return full paths or relative paths
        all_files = self.s3().find_files(bucket=self.s3_bucket(), prefix=folder)
        if full_path:
            return all_files
        else:
            if folder:
                return [file.replace(folder, '')[1:] for file in all_files]
        return []


    def s3_folder_list(self, folder='', return_full_path=False):
        return self.s3().folder_list(s3_bucket=self.s3_bucket(), parent_folder=folder, return_full_path=return_full_path)

    def s3_save_bytes(self,data, s3_key, metadata=None , content_type=None):
        kwargs = dict(bucket       = self.s3_bucket()                           ,
                      key          = s3_key                                     ,
                      file_body    = data                                       ,
                      metadata     = metadata                                   ,
                      content_type = str(content_type) if content_type else None)
        return self.s3().file_upload_from_bytes(**kwargs)

    def s3_save_data(self, data, s3_key, metadata=None, content_type=None):
        from osbot_utils.utils.Json import json_dumps, json_to_gz

        if self.save_as_gz:
            data      = json_to_gz(data)
        kwargs = dict(bucket=self.s3_bucket(), key=s3_key)
        if type(data) == bytes:
            kwargs['file_body'   ] = data
            kwargs['metadata'    ] = metadata
            kwargs['content_type'] = str(content_type) if content_type else None
            return self.s3().file_upload_from_bytes(**kwargs)
        else:
            data_as_str = json_dumps(data)
            kwargs["file_contents"] = data_as_str                   # todo: add support for capturing metadata
            return self.s3().file_create_from_string(**kwargs)

    # todo: refactor these s3_temp_folder__pre_signed_urls_* into a separate class (focused on pre_signed_urls)
    def s3_temp_folder__pre_signed_urls_for_object(self, source='NA', reason='NA', who='NA', expiration=3600):
        from osbot_utils.utils.Misc import random_guid, timestamp_utc_now

        s3_bucket          = self.s3_bucket__temp_data()
        s3_temp_folder     = self.s3_folder_temp_file_uploads()
        s3_object_name     = random_guid()
        s3_key             = f'{s3_temp_folder}/{s3_object_name}'
        pre_signed_url__get = self.s3_temp_folder__pre_signed_url(s3_bucket, s3_key, operation='get_object', expiration=expiration)
        pre_signed_url__put = self.s3_temp_folder__pre_signed_url(s3_bucket, s3_key, operation='put_object', expiration=expiration)
        pre_signed_data = dict(pre_signed_url__get = pre_signed_url__get ,
                               pre_signed_url__put = pre_signed_url__put ,
                               reason              = reason              ,
                               timestamp           = timestamp_utc_now() ,
                               source              = source              ,
                               who                 = who                 )
        return pre_signed_data

    def s3_temp_folder__pre_signed_url(self, s3_bucket, s3_key, operation,expiration=3600):
        create_kwargs = dict(bucket_name=s3_bucket,
                             object_name=s3_key,
                             operation=operation,
                             expiration=expiration)
        pre_signed_url = self.s3().create_pre_signed_url(**create_kwargs)
        return pre_signed_url

    def s3_temp_folder__pre_signed_url__download_string(self, pre_signed_url):
        import requests
        response = requests.get(pre_signed_url)
        if response.status_code == 200:
            return response.text
        #pprint(response)                   # todo: add a better way to handle the we dont' get an 200 status_code

    def s3_temp_folder__pre_signed_url__upload_string(self, pre_signed_url, file_contents):
        import requests

        response = requests.put(pre_signed_url, data=file_contents)
        if response.status_code == 200:
            return True
        else:
            return False

    def s3_folder_temp_file_uploads(self):
        return S3_FOLDER__TEMP_FILE_UPLOADS

    # setup and restore

    def bucket_delete(self):
        bucket_name = self.s3_bucket()
        return self.s3().bucket_delete(bucket_name)

    def bucket_delete_all_files(self):
        return self.s3().bucket_delete_all_files(self.s3_bucket())

    def bucket_exists(self):
        bucket_name = self.s3_bucket()
        return self.s3().bucket_exists(bucket_name)

    def setup(self):

        from osbot_aws.AWS_Config  import aws_config
        from osbot_utils.utils.Dev import pformat

        bucket_name = self.s3_bucket()

        kwargs = dict(bucket     = bucket_name              ,
                      region     = aws_config.region_name() ,
                      versioning = self.bucket_versioning   )

        if self.s3().bucket_not_exists(bucket_name):
            result = self.s3().bucket_create(**kwargs)
            if result.get('status') != 'ok':
                message = f"""Catastrophic error: Failed to create bucket:
                
                            { pformat(self.json()) }
                            
                            { pformat(result) }
                            """
                raise Exception(message)                                                                # todo: find a better way to handle these 'catastrophic' errors
            if self.bucket_versioning:                                                                  # if bucket versioning is enabled
                self.s3().bucket_versioning__enable(bucket_name)                                        #    configure it
            assert result.get('status') == 'ok'



        return self

    def using_minio(self):
        from osbot_aws.aws.s3.S3__Minio import DEFAULT__MINIO__SERVER

        return self.use_minio and self.s3().client().meta.endpoint_url == DEFAULT__MINIO__SERVER

    def using_local_stack(self):
        return self.s3().client().meta.endpoint_url == DEFAULT__LOCAL_STACK__TARGET_SERVER


from osbot_utils.type_safe.primitives.domains.files.safe_str.Safe_Str__File__Path import Safe_Str__File__Path
from osbot_utils.utils.Files                                                      import current_temp_folder
from osbot_aws.AWS_Config                                                         import AWS_Config
from osbot_aws.aws.lambda_.schemas.Safe_Str__File__Name__Python_Package           import Safe_Str__File__Name__Python_Package
from osbot_utils.type_safe.Type_Safe                                              import Type_Safe


DEFAULT__TEMP_FOLDER  = Safe_Str__File__Path(current_temp_folder())

class Lambda__Dependency__Base(Type_Safe):
    aws_config                  : AWS_Config
    temp_folder                 : Safe_Str__File__Path                  = DEFAULT__TEMP_FOLDER
    package_name                : Safe_Str__File__Name__Python_Package = None

    def __init__(self, package_name):
        super().__init__(package_name=package_name)                                 # make sure package name is defined





from unittest                                           import TestCase
from osbot_local_stack.local_stack.Local_Stack          import Local_Stack
from osbot_aws.AWS_Config                               import aws_config
from osbot_aws.aws.s3.S3                                import S3
from osbot_aws.testing.Temp__Random__AWS_Credentials    import Temp__Random__AWS_Credentials
from osbot_utils.utils.Misc                             import random_uuid_short


class TestCase__S3__Temp_Bucket(TestCase):

    @classmethod
    def setUpClass(cls):
        cls.local_stack = Local_Stack().activate()
        assert cls.local_stack.is_local_stack_configured_and_available() is True
        cls.random_aws_creds  = Temp__Random__AWS_Credentials().set_vars()
        cls.temp_bucket_name  = 'temp-bucket-' + random_uuid_short()
        cls.versioning        = True
        cls.region_name       = aws_config.region_name() or "eu-east-1"         #todo: see why we need to default to eu-east-1 in GH actions
        cls.s3                = S3()
        assert cls.s3.bucket_create(cls.temp_bucket_name, region=cls.region_name).get('status') is 'ok'
        assert cls.s3.bucket_exists(cls.temp_bucket_name)                                       is True

    @classmethod
    def tearDownClass(cls):
        assert cls.local_stack.is_local_stack_configured_and_available() is True
        assert cls.s3.bucket_delete_all_files(cls.temp_bucket_name) is True
        assert cls.s3.bucket_delete          (cls.temp_bucket_name) is True
        cls.random_aws_creds.restore_vars()
        cls.local_stack.deactivate()